/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2012 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.admin.storage.sqlStorage;

import java.sql.Connection;
import java.sql.SQLException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import com.openexchange.admin.rmi.exceptions.PoolException;
import com.openexchange.admin.tools.PropertyHandler;
import com.openexchange.database.DBPoolingException;
import com.openexchange.databaseold.Database;

public class OXAdminPoolDBPool implements OXAdminPoolInterface {

    private Log log = LogFactory.getLog(this.getClass());

    public OXAdminPoolDBPool(PropertyHandler prop) {
        super();
    }

    public Connection getConnectionForConfigDB() throws PoolException {
        final Connection con;
        try {
            con = Database.get(true);
        } catch (DBPoolingException e) {
            log.error("Error pickup configdb database write connection from pool!", e);
            throw new PoolException(e.getMessage());
        }
        return con;
    }

    public Connection getConnectionForContext(int contextId) throws PoolException {
        final Connection con;
        try {
            con = Database.get(contextId, true);
        } catch (DBPoolingException e) {
            log.error("Error pickup context database write connection from pool!", e);
            throw new PoolException(e.getMessage());
        }
        return con;
    }

    public Connection getConnectionForContextNoTimeout(int contextId) throws PoolException {
        final Connection con;
        try {
            con = Database.getNoTimeout(contextId, true);
        } catch (DBPoolingException e) {
            log.error("Error pickup context database write connection from pool!", e);
            throw new PoolException(e.getMessage());
        }
        return con;
    }

    public boolean pushConnectionForConfigDB(Connection con) throws PoolException {
        try {
            if (con != null && !con.getAutoCommit() && !con.isClosed()) {
                con.setAutoCommit(true);
            }
        } catch (SQLException e) {
            log.error("Error pushing configdb write connection to pool!", e);
            throw new PoolException(e.getMessage());
        } finally {
            Database.back(true, con);
        }
        return true;
    }

    public boolean pushConnectionForContext(int contextId, Connection con) throws PoolException {
        try {
            if (con != null && !con.getAutoCommit() && !con.isClosed()) {
                con.setAutoCommit(true);
            }
        } catch (SQLException e) {
            log.error("Error pushing context database write connection to pool!", e);
            throw new PoolException(e.getMessage());
        } finally {
            if (null != con) {
                Database.back(contextId, true, con);
            }
        }
        return true;
    }

    public boolean pushConnectionForContextNoTimeout(int contextId, Connection con) throws PoolException {
        try {
            if (null != con && !con.getAutoCommit() && !con.isClosed()) {
                con.setAutoCommit(true);
            }
        } catch (SQLException e) {
            log.error("Error pushing context database write connection to pool!", e);
            throw new PoolException(e.getMessage());
        } finally {
            Database.backNoTimeout(contextId, true, con);
        }
        return true;
    }

    public int getServerId() throws PoolException {
        final int serverId;
        try {
            serverId = Database.getServerId();
        } catch (DBPoolingException e) {
            log.error("Error getting the identifier of the server! This is normal until at least one server is configured.", e);
            throw new PoolException(e.getMessage());
        }
        return serverId;
    }
}
